

(* File: NameMap.m3                                            *)
(* Last modified on Fri Nov  6 16:30:21 MET 1992 by preschern  *)

MODULE NameMap;

IMPORT TxtTxtTbl, FileIO, Filename, Text, Rd, Wr, Stdio, Fmt, Scan, List,
       DOSTextRd, DOSTextWr;

CONST
  MAPNAME = "NAMES.MAP";
  MAPLEN  = 8;
  MAPMAX  = 99999999;

REVEAL T = BRANDED "NameMap.T" OBJECT
             dir  : TEXT;
             count: INTEGER;
             table: TxtTxtTbl.T;
           END (* object *);

PROCEDURE New (dir: TEXT) : T =
  VAR new: T;
  BEGIN
    new       := NEW (T);
    new.dir   := dir;
    new.count := 0;
    new.table := TxtTxtTbl.New (); 
    RETURN new;
  END New;

PROCEDURE Exists (map: T) : BOOLEAN =
  VAR rd: Rd.T;
      filename: TEXT;
  BEGIN
    filename := map.dir & "/" & MAPNAME;
    TRY
      rd := FileIO.OpenRead (filename);
      Rd.Close (rd);
    EXCEPT ELSE
      RETURN FALSE;
    END (* try *);
    RETURN TRUE;
  END Exists;

PROCEDURE ReadNameMap (map: T) =
  VAR rd: Rd.T;
      filename: TEXT;
      line, long, short: TEXT;
  BEGIN
    filename := map.dir & "/" & MAPNAME;
    TRY
      rd        := DOSTextRd.New (FileIO.OpenRead (filename));
      line      := Rd.GetLine (rd); 
      map.count := Scan.Int (line); 
      WHILE NOT Rd.EOF (rd) DO
        long  := Rd.GetLine (rd);
        short := Rd.GetLine (rd); 
        IF map.table.put (long, short) THEN
          Wr.PutText (Stdio.stdout, "Sorry, dublicate entry in " & 
                                     filename & "\n");
          <* ASSERT FALSE *> 
        END (* if *);
      END (* while *);
      Rd.Close (rd);
    EXCEPT ELSE
      TRY Wr.PutText (Stdio.stdout, "Sorry, cannot open " & filename & "\n");
      EXCEPT ELSE
      END (* try *);
    END (* try *);
  END ReadNameMap;

PROCEDURE WriteNameMap (map: T) =
  VAR wr: Wr.T;
      filename: TEXT;
      key: TxtTxtTbl.Key;
      value: TxtTxtTbl.Value;

  PROCEDURE WriteOut (    <* UNUSED *> data : REFANY; 
                          key  : TxtTxtTbl.Key; 
                      VAR value: TxtTxtTbl.Value): BOOLEAN =
    BEGIN
      TRY 
        Wr.PutText (wr, key & "\n");
        Wr.PutText (wr, value & "\n");
      EXCEPT ELSE
        TRY Wr.PutText (Stdio.stdout, "Sorry, cannot write " & filename & "\n");
        EXCEPT ELSE 
        END (* try *);
      END (* try *);
      RETURN FALSE;
    END WriteOut;

  BEGIN
    <* ASSERT map # NIL *>
    filename := map.dir & "/" & MAPNAME;
    TRY
      wr := DOSTextWr.New (FileIO.OpenWrite(filename)); 
      Wr.PutText (wr, Fmt.Int (map.count) & "\n");
      EVAL map.table.enumerate (WriteOut, NIL, key, value);
      Wr.Close (wr);
    EXCEPT ELSE
      TRY Wr.PutText (Stdio.stdout, "Sorry, cannot create " & filename & "\n");
      EXCEPT ELSE 
      END (* try *);
    END (* try *); 
  END WriteNameMap;

PROCEDURE EnterInMap (map: T; name: TEXT): BOOLEAN =
  VAR long, short: TEXT;
  BEGIN
    IF (map = NIL) OR (name = NIL) THEN 
      RETURN FALSE 
    END (* if *);
    long := name;
    INC (map.count); <* ASSERT map.count <= MAPMAX *>
    IF (Text.Length (Filename.Root (name)) > MAPLEN) THEN
      short:= Fmt.Int (map.count) & "." & Filename.Extension (name);
    ELSE
      short := name;
    END (* if *);
    RETURN NOT map.table.put (long, short);
  END EnterInMap;

PROCEDURE EnterUntruncated (map: T; name: TEXT): BOOLEAN =
  BEGIN
    IF (map = NIL) OR (name = NIL) THEN 
      RETURN FALSE 
    END (* if *);
    RETURN NOT map.table.put (name, name);
  END EnterUntruncated;

PROCEDURE RemoveFromMap (map: T; name: TEXT): BOOLEAN =
  VAR value: TxtTxtTbl.Value;
  BEGIN
    IF (map = NIL) OR (name = NIL) THEN 
      RETURN FALSE 
    END (* if *);
    INC (map.count); <* ASSERT map.count <= MAPMAX *>
    IF map.table.delete (name, value) THEN
      <* ASSERT map.count > 0 *> DEC (map.count);
      RETURN TRUE;
    END (* if *);
    RETURN FALSE;
  END RemoveFromMap;

PROCEDURE GetMapping (map: T; name: TEXT; VAR value: TEXT) : BOOLEAN =
  BEGIN
    IF (map = NIL) OR (name = NIL) THEN
      RETURN FALSE 
    END (* if *);
    RETURN map.table.in (name, value);
  END GetMapping;

PROCEDURE GetDir (map: T) : TEXT =
  BEGIN
    IF (map = NIL) THEN RETURN NIL END (* if *);
    RETURN map.dir;
  END GetDir;

PROCEDURE ToList (map: T): List.T =
  BEGIN
    IF (map = NIL) OR (map.table = NIL) THEN RETURN NIL END (* if *);
    RETURN map.table.toKeyList (); 
  END ToList;

PROCEDURE Lookup (n: TEXT): TEXT =
  VAR result, head, tail: TEXT;
      map: T;
  BEGIN
    head := Filename.Head (n);
    tail := Filename.Tail (n);
    IF (Text.Length (head) = Text.Length (n)) THEN head := "." END (* if *);
    result := tail;
    map := New (head);
    IF Exists (map) THEN
      ReadNameMap (map);
    END (* if *); 
    IF NOT GetMapping (map, tail, result) THEN result := tail END (* if *);
    result := head &  "/" & result;
    RETURN result;
  END Lookup;

PROCEDURE Lookdown (n: TEXT): TEXT =
  VAR result, head, tail: TEXT;
      map: T;
  BEGIN
    head := Filename.Head (n);
    tail := Filename.Tail (n);
    IF (Text.Length (head) = Text.Length (n)) THEN head := "." END (* if *);
    result := tail;
    map := New (head);
    IF Exists (map) THEN
      ReadNameMap (map);
    END (* if *); 
    IF NOT GetMapping (map, tail, result) THEN
      EVAL EnterInMap (map, tail); 
      EVAL GetMapping (map, tail, result);
      WriteNameMap (map);             (* stay in sync with map on disk *)
    END (* if *);
    result := head &  "/" & result;
    RETURN result;
  END Lookdown;

PROCEDURE Delete (n: TEXT): BOOLEAN =
  VAR head, tail: TEXT;
      map: T;
  BEGIN
    head := Filename.Head (n);
    tail := Filename.Tail (n);
    IF (Text.Length (head) = Text.Length (n)) THEN head := "." END (* if *);
    map := New (head);
    IF Exists (map) THEN
      ReadNameMap (map);
    ELSE
      RETURN FALSE;
    END (* if *); 
    IF RemoveFromMap (map, tail) THEN
      WriteNameMap (map);
      RETURN TRUE;
    END (* if *);
    RETURN FALSE; 
  END Delete;

BEGIN
END NameMap.

